/**
 *  File name: "entry.h"
 *  This file contains class 'Entry' that holds the information of an 
 *  entry in the table.
 * 
 *  Created on 25-Apr-2006
 * 
 *  Copyright (C) 2005-2006 the MAGIC team.
 *  Author(s): Rani Zand <zanood@gmail.com>.
 *
 *
 *  This file is part of MAGIC, an Integrative and Accurate Comparative
 *  Genome Mapping tool.
 *
 *
 *  MAGIC is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GCC; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef ENTRY_H
#define ENTRY_H

#define LABEL_SIZE 90



/*************************************************************/
/*********************** Class Entry *************************/
/*************************************************************/

#include <string>

using namespace std;

/**
 * Class name: Entry
 *
 * This class is used to hold the information of an entry in the table.
 * The information that is held is:
 * 1) The satrts and ends of the two segments.
 * 2) The Sign of the entry, '+' means the two segments are have the same direction,
 *    '-' means they have opposite directions.
 * 3) The label of the entry.
 *
 * @author Rani Zand
 *
 */
template <class GENUM_TYPE>
class Entry{

private:
	/**
	 * Indicates if the entry is used or not.
	 */
	bool InUse; 

	/**
	 * The entry's label.
	 */
	string Label;

	/**
	 * Starts and Ends of organisms 1 and 2.
	 */
	GENUM_TYPE S1 , E1 , S2 , E2 ; // start/end of genum 1/2

	/**
	 * Entry's sign, if Sign = true that means the the direction of [S1,E1] and [S2,E2] are
	 * the same, if Sign = false the directions are opposite.
	 */
	bool Sign; 

public:

	/**
	 * Constructor.
	 */
	Entry(): InUse(0) {}

	/**
	 * Copy constructor.
	 */
	Entry(const Entry<GENUM_TYPE>& ent);

	/**
	 * Distructor.
	 */
	~Entry(){}

	/**
	 * Sets the entry's parameters.
	 * @param s_1 the start of the first segment of the entry.
	 * @param e_1 the end of the first segment of the entry.
	 * @param s_2 the start of the second segment of the entry.
	 * @param e_2 the end of the second segment of the entry.
	 * @param sign_ the sign of the entry.
	 * @param label_ the label of the entry.
	 */
	void InseretEntry(GENUM_TYPE s_1 ,GENUM_TYPE e_1 ,GENUM_TYPE s_2 ,GENUM_TYPE e_2 ,bool sign_ ,const string label_);

	/**
	 * Gets the start and end of the first segment.
	 * @param start reference to a variable the its value will be the start of the first segment.
	 * @param end reference to a variable the its value will be the end of the first segment.
	 */
	void GetS1E1(GENUM_TYPE& start ,GENUM_TYPE& end)	const ;

	/**
	 * Gets the start and end of the second segment.
	 * @param start reference to a variable the its value will be the start of the second segment.
	 * @param end reference to a variable the its value will be the end of the second segment.
	 */
	void GetS2E2(GENUM_TYPE& start ,GENUM_TYPE& end)	const ;

	/**
	 * Sets the start and end of the first segment.
	 * @param start the new start of the first segment.
	 * @param end the new end of the first segment.
	 */
	void SetS1E1(GENUM_TYPE start ,GENUM_TYPE end);

	/**
	 * Sets the start and end of the second segment.
	 * @param start the new start of the second segment.
	 * @param end the new end of the second segment.
	 */
	void SetS2E2(GENUM_TYPE start ,GENUM_TYPE end);

	/**
	 * Return the sign of the entry, true if the two segments have the
	 * same direction, false otherwise.
	 * @return the sign of the entry.
	 */
	bool GetSign() const ;

	/**
	 * Return the label of the entry.
	 * @return the label of the entry.
	 */
	string GetLabel() const ;

	/**
	 * Return true if the entry is in use.
	 * @return true if the entry is in use.
	 */
	bool IsInUse() const ;

	/**
	 * Sets the parameter InUse to false
	 */
	void Delete() ;

	/**
	 * operator =, copys all the variables of 'ent' to this .
	 */
	Entry<GENUM_TYPE>& operator=(const Entry<GENUM_TYPE>& ent);

	/**
	 * operator ==, return true if the two entries have the same label .
	 */
	bool operator==(const Entry<GENUM_TYPE>& ent) const;
};

/*************************************************************/
/******************** Class Entry Mothods ********************/
/*************************************************************/

template <class GENUM_TYPE>
void Entry<GENUM_TYPE>::InseretEntry(GENUM_TYPE s_1 ,GENUM_TYPE e_1 ,GENUM_TYPE s_2 ,GENUM_TYPE e_2 ,bool sign_ ,const string label_)	{
	S1=s_1; E1=e_1 ; S2=s_2 ; E2=e_2 ; Sign=sign_ ; Label = label_;
	this->InUse = true;
}

/*************************************************************/

template <class GENUM_TYPE>
void Entry<GENUM_TYPE>::GetS1E1(GENUM_TYPE& start ,GENUM_TYPE& end) const 	{
	start = this->S1 ; end = this->E1;
}

/*************************************************************/

template <class GENUM_TYPE>
void Entry<GENUM_TYPE>::GetS2E2(GENUM_TYPE& start ,GENUM_TYPE& end) const  {
	start = this->S2 ; end = this->E2;
}

/*************************************************************/

template <class GENUM_TYPE>
void Entry<GENUM_TYPE>::SetS1E1(GENUM_TYPE start ,GENUM_TYPE end) {
	this->S1 = start ; this->E1 = end;
}

/*************************************************************/

template <class GENUM_TYPE>
void Entry<GENUM_TYPE>::SetS2E2(GENUM_TYPE start ,GENUM_TYPE end) {
	this->S2 = start ; this->E2 = end;
}

/*************************************************************/

template <class GENUM_TYPE>
bool Entry<GENUM_TYPE>::GetSign() const {
	return this->Sign ;
}

/*************************************************************/

template <class GENUM_TYPE>
string Entry<GENUM_TYPE>::GetLabel() const {
	string tmp = Label;
	return tmp;
}

/*************************************************************/

template <class GENUM_TYPE>
bool Entry<GENUM_TYPE>::IsInUse() const {
	return this->InUse;
}

/*************************************************************/

template <class GENUM_TYPE>
void Entry<GENUM_TYPE>::Delete() {
	this->InUse = false;
}

/*************************************************************/

template <class GENUM_TYPE>
Entry<GENUM_TYPE>& Entry<GENUM_TYPE>::operator=(const Entry<GENUM_TYPE>& ent){
	this->E1 = ent.E1;	this->S1 = ent.S1;
	this->E2 = ent.E2;	this->S2 = ent.S2;
	this->InUse = ent.InUse;
	this->Sign = ent.Sign ;
	this->Label = ent.Label;

	return *this;
}

/*************************************************************/

template <class GENUM_TYPE>
Entry<GENUM_TYPE>::Entry(const Entry<GENUM_TYPE>& ent){
	this->E1 = ent.E1;	this->S1 = ent.S1;
	this->E2 = ent.E2;	this->S2 = ent.S2;
	this->InUse = ent.InUse;
	this->Sign = ent.Sign ;
	this->Label = ent.Label;

}

/*************************************************************/

template <class GENUM_TYPE>
bool Entry<GENUM_TYPE>::operator==(const Entry<GENUM_TYPE>& ent) const{
	return (this->Label == ent.Label);
}


#endif
