/**
 *  File name: "table.h"
 *  This file contains class 'Table' that holds the information of the table. 
 *
 *  Created on 25-Apr-2006
 * 
 *  Copyright (C) 2005-2006 the MAGIC team.
 *  Author(s): Rani Zand <zanood@gmail.com>.
 *
 *
 *  This file is part of MAGIC, an Integrative and Accurate Comparative
 *  Genome Mapping tool.
 *
 *
 *  MAGIC is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GCC; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 */


#ifndef TABLE_H
#define TABLE_H

#include "entry.h"
#include "genome.h"
#include "compare_entries.h"
#include <iostream>
#include <algorithm>
#include <fstream>
#include <string>
#include <set>

using namespace std;

/**
 * When this Macro's value is 1, the deleted entries in the RemoveParalogs step are printed
 * to two given files. when the Macro's value is 0, the deleted entries arent deleted.
 * The reason of making a macro for this is because these files arent used yet in the software.
 */
#define PRINT_DELETED_PARALOGS 1

//*************************************************************/
//*********************** Class Table *************************/
//*************************************************************/

/**
 * Class name: Table
 *
 * This class hold the information of the table, and all the operations
 * that are made on the table.
 * The information includes:
 * 1) An array of entries.
 * 2) Number of entries in the table.
 * 3) information about the first genome (of type Genome).
 * 4) information about the second genome (of type Genome).
 *
 * @author Rani Zand
 *
 */
template <class GENOME_TYPE>
class Table{

private:
	/**
	 * The entries of the tables.
	 */
	Entry<GENOME_TYPE>* entries;

	/**
	 * Number of entries that are in use in the table.
	 */
	unsigned long numOfEntries;

	/**
	 * Number of entries in the table (including unused entries).
	 */
	unsigned long tableSize;

	/**
	 * The number of the first entry that is circular in organism 1.
	 */
	unsigned long firstCircOrg1;
	
	/**
	 * The number of the first entry that is circular in organism 2.
	 */
	unsigned long firstCircOrg2;
	
	/**
	 * The number of the last entry that is circular in organism 1.
	 */
	unsigned long lastCircOrg1;
	
	/**
	 * The number of the last entry that is circular in organism 2.
	 */
	unsigned long lastCircOrg2;

	/**
	 * Information about the first organism.
	 */
	Genome<GENOME_TYPE> organism1;
	
	/**
	 * Information about the second organism.
	 */
	Genome<GENOME_TYPE> organism2;

	/**
	 * A parameter used to delete all entries that their length is smaller that minLen/
	 */
	GENOME_TYPE minLen;

	/**
	 * Used to print the deleted entries in organism 1 from the RemoveParalog step.
	 */
	Entry<GENOME_TYPE> *deletedParalog1;

	/**
	 * Used to print the deleted entries in organism 2 from the RemoveParalog step.
	 */
	Entry<GENOME_TYPE> *deletedParalog2;

	/**
	 * Counter of the number of entries that were deleted in the RemoveParalog step in organism 1.
	 */
	unsigned long deletedParalogCount1;

	/**
	 * Counter of the number of entries that were deleted in the RemoveParalog step in organism 2.
	 */
	unsigned long deletedParalogCount2;
	
	/**
	 * A parameter used to find duplicated entries.
	 */
	double dupPerc;

	/**
	 * Used to scan a set.
	 */
	typedef typename std::set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> >::iterator entryIter;	

	/**
	 * This method gets an entries file and inserts the entries in the table.  
	 * @param fd the file that contains the entries.
	 */
	void InsertEntries(ifstream& fd) ;

	/**
	 * Swap entry i with entry j.  
	 * @param i number of first entry to swap.
	 * @param j number of second entry to swap.
	 */
	void SwapEntries(unsigned long i , unsigned long j) ;

	/**
	 * Delete entry i from the table  
	 * @param i the entry to delete.
	 */
	void DeleteEntry(unsigned long i) ;

	/**
	 * Sort the table according to 'org'.  
	 * @param org the number of the organism to sort according to  
	 */
	void SortBy(int org); 

	/**
	 * Return the percentage of the intersection between 'entry' and segment [S,E] in organism 'orgNum'. 
	 * if they dont intersect, 0 is returned.
	 * @param entry the entry working on.
	 * @param S the start of the segment.
	 * @param E the end of the segment.
	 * @param orgNum orgNum the number of the organism.
	 * @return the percentage of intersection.
	 */
	double Intersec(unsigned long entry, GENOME_TYPE S, GENOME_TYPE E,int orgNum) const;

	/**
	 * Return the intersection length between segment [S,E] and segment [tmp_S,tmp_E] in organism 'orgNum'.  
	 * @param S the start of the segment.
	 * @param E the end of the segment.
	 * @param tmpS the start of the segment.
	 * @param tmpE the end of the segment.
	 * @param orgNum orgNum the number of the organism.
	 * @return the intersection length
	 */
	GENOME_TYPE InersecLen(GENOME_TYPE S, GENOME_TYPE E, GENOME_TYPE tmpS,
										 GENOME_TYPE tmpE , int orgNum) const;
	
	/**
	 * Return the length of segment [S,E] in organism 'orgNum'  
	 * @param S the start of the segment.
	 * @param E the end of the segment.
	 * @param orgNum orgNum the number of the organism.
	 * @return the length of [S,E].
	 */
	GENOME_TYPE GetLength(GENOME_TYPE S, GENOME_TYPE E, int orgNum ) const;

	/**
	 * Used in method Intersec. if one of the segments [S1,E1] or [S2,E2] is circular we add the organism's
	 * length to some of S1,E1,S2,E2 accoding to the intersection of both segments in order to calculate
	 * the percentage of intersection in method Intersec.
	 * @param S1 the start of the first segment.
	 * @param E1 the end of the first segment.
	 * @param S2 the start of the second segment.
	 * @param E2 the end of the second segment.
	 */
	void ManipulateStartsAndEnds(GENOME_TYPE& S1, GENOME_TYPE& E1, GENOME_TYPE& S2, GENOME_TYPE& E2,int orgNum) const;
	
	/**
	 * This method returns true if 'entry' is a duplicate of 'ofEntry' in 'orgNum', false if not.  
	 * @param entry the first entry.
	 * @param ofEntry the second entry.
	 * @param orgNum orgNum the number of the organism.
	 * @return true if 'entry' is a duplicate of 'ofEntry' in 'orgNum', false if not.
	 */
	bool IsDuplication(unsigned long entry , unsigned ofEntry , int orgNum ) const;
	
	/**
	 * Return a set of the entries that are duplicates of entry 'entry'.
	 * In order to find the duplications of entry 'entry' we used and array 'endArr' that
	 * its size is the same as the size of the table. The value of each entry in 'endArr' means that
	 * for this entry and all smaller entries in the table, that value is the bigget E of all entries in
	 * organism 'orgNum'.  
	 * assuming the table is sorted according to 'orgNum'.  
	 * @param entry the entry that its duplications are returned.
	 * @param orgNum the number of the organism.
	 * @param endArr the array used to find the duplications.
	 * @return a set of the duplication entries.
	 */
	set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> > GetDupGroup(unsigned long entry , int orgNum, GENOME_TYPE *endArr) ;

	/**
	 * This method updates the vaues of 'arr' which is an array used to find the duplicates 
	 * of an entry. (more details are found in function 'GetDupGroup').
	 * @param arr the array that is updated.
	 * @param orgNum the number of the organism.
	 */
	void FillEnds(GENOME_TYPE *arr,int orgNum) const;

	/**
	 * Search for entry S1,E1,S2,E2 and return th index , assuming table is sorted by 'orgNum'.
	 * @param S1 the start of the first segment.
	 * @param E1 the end of the first segment.
	 * @param S2 the start of the second segment.
	 * @param E2 the end of the second segment.
	 * @param orgNum the number of the organism.
	 * @return the index of the entry.
	 */
	unsigned long BinarySearch(GENOME_TYPE S1,GENOME_TYPE E1,GENOME_TYPE S2,GENOME_TYPE E2,int orgNum)const ;

	/**
	 * This method gets an array 'arr' that is as big as the table. The array contains numbers of entry 
	 * in the table, if arr[i] = j that means that entry 'i' will be joined with entry 'j'.
	 * @param arr the array of indexes
	 */
	void Join(unsigned long *arr) ;
	
	/**
	 * This method filters all entries of the set 's' that are duplicate to entry 'j' 
	 * in organism 'org'.	
	 * If 'flag' = true , it removes all the entries of the set 's' that their sign isnt 'sign'. 
	 * @param s the Set to filter.
	 * @param j the entry that 's' is filtered according to it.
	 * @param org the number of the organism.
	 * @param flag if true then filter also according to 'sign', if false not.
	 * @param sign the sign to filter according to if flag = true.
	 */
	void FilterSet(set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> >& s, unsigned long j, int org, bool flag, bool sign=false) ;

	/**
	 * If one of the organisms is circular, this method updates the value of 'firstCircOrg1', 'firstCircOrg2',
	 * 'lastCircOrg1' and 'lastCircOrg1'.
	 */
	void UpDateCircInfo() ;

	/**
	 * This method is used in the 'RemoveOverlaps' method. It gets two entries, 'i' and 'j', and
	 * checks if they intersect in both organisms, if so it joins them to one entry and return true,
	 * if not return false. 
	 * @param i the first entry.
	 * @param j the second entry.
	 * @return true if joined, false if not.
	 */
	bool JoinForOverlap(unsigned long i, unsigned long j);

	/**
	 * This method gets a .prop file 'fileName' and gets from it the genome's length and circularity.
	 * @param fileName the .prop file.
	 * @param genumLen reference to the genome's length.
	 * @param circ reference to the genome's circularity.
	 */
	void ReadFromProbFile(ifstream& fileName, string &genumName, GENOME_TYPE &genumLen, bool &circ);

	/**
	 * Print the deleted paralogs from both organisms to 'file1' and 'file2'.
	 * Note: deleted paralogs the are intersected with percentage more than 0.7 are joined.
	 * @param file1 the first file.
	 * @param file2 the second file.
	 */
	void PrintDeletedParalogs(ofstream& file1, ofstream& file2);

	/**
	 * If entry 'i' and entry 'j' are intersected in organism 'orgNum' with percentage more 
	 * than 0.7 these entries are joined to one entry and true is returned, if not nothing is done
	 * and false is returned.
	 * @param i the first entry.
	 * @param j the second entry.
	 * @param orgNum the number of the organism.
	 * @return true if joined, false if not.
	 */
	bool JoinForDeletedParalogs(unsigned long i, unsigned long j , int orgNum);
	

public:
	/**
	 * Constructor.
	 */
	Table(){}

	/**
	 * Constructor.
	 * @param fd the file that contains the entries of the table.
	 * @param numOfRows number of rows in the table.
	 * @param cleanMinLen a parameter used in 'CleanShortEntries' method.
	 * @param dupPerc a parameter used in 'IsDuplication' method.
	 * @param org1DataFile the path of "org1".prob
	 * @param org2DataFile the path of "org2".prob
	 */
	Table(ifstream& fd, long numOfRows,long cleanMinLen,double dupPerc, ifstream& orag1DataFile, ifstream& org2DataFile) ;

	/**
	 * Copy constructor.
	 * @param t the table to copy from.
	 */
	Table(const Table<GENOME_TYPE>& t);

	/**
	 * Distructor.
	 */
	~Table() { 
		delete[] entries;
	}
	
	/**
	 * Clean duplicated entries (same S,E in both organisms) from the table.
	 */
	void CleanDup(); 	

	/**
	 * Clean entries that their length is one of the organisms at least is smaller thab 'minLen'.
	 */
	void CleanShortEntries();

	/**
	 * Remove the unused entries from the table. 
	 */
	void CleanDeleted();	

	/**
	 * This method gets a file 'fd' that contains S and Es of prophages, and deletes all
	 * the entries that the percentage of intersection with these S and Es (sum for all prophages)
	 * in organism 'orgNum' is bigger than 'perc'.
	 * @param fd the prophages file.
	 * @param orgNum the number of the organism to work on.
	 * @param perc the percentage of intersection that according to it entries are deleted.
	 */
	void CleanProphages(ifstream& fd ,int orgNum, double perc);

	/**
	 * This method gets a file 'fd' that contains S and Es of transposable elements, and deletes all
	 * the entries that the percentage of intersection with these S and Es (sum for all tansposable elements)
	 * in organism 'orgNum' is bigger than 'perc'.
	 * @param fd the transposable elements file.
	 * @param orgNum the number of the organism to work on.
	 * @param perc the percentage of intersection that according to it entries are deleted.
	 */
	void CleanTE(ifstream& fd ,int orgNum, double perc);

	/**
	 * Prints the table to file 'ofs' sorted according to 'org'.
	 * @param ofs the file to print the table to.
	 * @param org the number of the organism that according to it the table is sorted.
	 */
	void PrintTable(ofstream& ofs,int org) ;

	/**
	 * This method makes the Clustering step that is explained........ 
	 */
	void Clustering();

	/**
	 * Check for each entry 'i' in the table if there is an entry 'j' such that 'i' and 'j'
	 * intersect in both organisms, if so 'i' and 'j' are joined into one entry.
	 */
	void RemoveOverlaps();

	/**
	 * This method keep on running 'RemoveOverlaps' method until there are no more entries to join.
	 */
	void IterativeRemoveOverlaps();

	/**
	 * For each entry 'i' we get the duplications of it in organism 1, s1, and the duplications 
	 * of it in organism 2, s2. if there is an entry 'j' in s1 such that length(i)/length(j) <= 'orthPerc'
	 * and there is an entry 'k' in s2 such that length(i)/length(k) <= 'orthPerc', then entry 'i' is
	 * deleted from the table.
	 */
	void RemoveNuisances(double orthPerc);

	/**
	 * For each entry 'i' we get the duplications of it in organism 1, s1, and the duplications 
	 * of it in organism 2, s2. if there is an entry 'j' in s1 such that length(i)/length(j) <= 'orthPerc'
	 * or there is an entry 'k' in s2 such that length(i)/length(k) <= 'orthPerc', then entry 'i' is
	 * deleted from the table.
	 */
	void RemoveParalogs(double orthPerc);

	/**
	 * This method keeps on running 'Clustering', 'RemoveNuisances' and 'RemoveParalogs' methods 
	 * until convergence of the table.
	 * @param orthPerc the percentage for the Paralog step.
	 * @param deletedParFile1 the file that will contain the deleted entries of the paralog step in organism 1.
	 * @param deletedParFile2 the file that will contain the deleted entries of the paralog step in organism 2.
	 */
	void Loop(double orthPerc,const char* deletedParFile1,const char* deletedParFile2);

	/**
	 * Every two entries that one is duplication of the other in one of the organisms, both are
	 * deleted from the table.
	 */
	void RemoveFinalConflicts();

	/**
	 * For each entry 'i', if there is an entry 'j' such that both entries intersect in one
	 * of the organisms, one of the entries is made smaller in both organisms (in the same percentage)
	 * so that no more intersection exsists between 'i' and 'j'.
	 */ 
	void RemoveFinalOverlaping();

};

//*************************************************************/
//******************** Class Table Mothods ********************/
//*************************************************************/

template <class GENOME_TYPE>
Table<GENOME_TYPE>::Table(ifstream& fd , long numOfRows,long cleanMinLen, double duplicatePerc, ifstream& org1DataFile, ifstream& org2DataFile) {
		tableSize = numOfRows; 
		GENOME_TYPE gen1Len , gen2Len ; 
		string gen1Name , gen2Name ;
		bool gen1Circ  , gen2Circ  ; 

		this->entries = new Entry<GENOME_TYPE>[this->tableSize];
		this->numOfEntries = 0 ;

		this->ReadFromProbFile(org1DataFile, gen1Name, gen1Len, gen1Circ);
		this->ReadFromProbFile(org2DataFile, gen2Name, gen2Len, gen2Circ);
		
		this->minLen = cleanMinLen;	
		this->dupPerc = duplicatePerc;
		this->organism1.SetGenomeLen(gen1Len);
		this->organism2.SetGenomeLen(gen2Len);
		this->organism1.SetGenomeName(gen1Name);
		this->organism2.SetGenomeName(gen2Name);
		this->organism1.SetGenomeCircular(gen1Circ);
		this->organism2.SetGenomeCircular(gen2Circ);
		this->InsertEntries(fd);
}

//*************************************************************/

template <class GENOME_TYPE>
Table<GENOME_TYPE>::Table(const Table<GENOME_TYPE>& t) {
	
	numOfEntries = t.numOfEntries;
	tableSize = t.tableSize;
	organism1 = t.organism1;
	organism2 = t.organism2;
	minLen = t.minLen;
	dupPerc = t.dupPerc;
	
	entries = new Entry<GENOME_TYPE>[t.tableSize];
	for(unsigned long i=0 ; i < this->tableSize ; i++) 
		entries[i] = t.entries[i];
}

//************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::InsertEntries(ifstream& fd) { 
	string st;
	GENOME_TYPE S1 , E1 , S2 , E2 ; 
	bool Sign;
	unsigned long i(0);

	getline(fd, st);
	getline(fd, st);
		
	i = 0;

	while ( i < this->tableSize ) {
		fd >> S1;
		fd >> E1;
		fd >> st ; // ( we dont need it )
		fd >> S2;
		fd >> E2;
		
		fd >> st ; // ( we dont need it )
		fd >> st ;
		if ( st.at(0) == '+' ) Sign = true;
		else Sign = false ;
		fd >> st ;
		
		this->entries[i++].InseretEntry(S1,E1,S2,E2,Sign,st);
		this->numOfEntries++;
	}
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::DeleteEntry(unsigned long i) {

	if ( !this->entries[i].IsInUse() ) return;
	this->entries[i].Delete();
	this->numOfEntries--;
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::SortBy(int orgNum){
	if (orgNum == 1 ) {
		sort(this->entries,this->entries+this->tableSize,compare_entries_1<GENOME_TYPE>());
	}
	else if (orgNum == 2 ) {
		sort(this->entries,this->entries+this->tableSize,compare_entries_2<GENOME_TYPE>());
	}
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::CleanDup(){  
	GENOME_TYPE last_start1 = -1 , last_end1 = -1 , curr_start1 , curr_end1 ,last_start2 = -1 , last_end2 = -1 , curr_start2 , curr_end2 ;

	this->SortBy(1);
	for(unsigned long i = 0 ; i < this->tableSize ; i ++) {
		if (!this->entries[i].IsInUse()) continue;
		this->entries[i].GetS1E1(curr_start1 , curr_end1);
		this->entries[i].GetS2E2(curr_start2 , curr_end2);
		if ( (curr_start1 == last_start1) && (curr_end1 == last_end1) && (curr_start2 == last_start2) && (curr_end2 == last_end2) )
			this->DeleteEntry(i);
		else {
			last_start1 = curr_start1;
			last_end1 = curr_end1 ;
			last_start2 = curr_start2;
			last_end2 = curr_end2 ;
		}
	}			
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::CleanShortEntries() {
	GENOME_TYPE S1,E1,S2,E2;
	for ( unsigned long i=0 ; i < this->tableSize ; i++) {
		if ( !entries[i].IsInUse() ) continue;
		this->entries[i].GetS1E1(S1,E1);
		this->entries[i].GetS2E2(S2,E2);

		if ( (this->GetLength(S1,E1,1) < this->minLen) || 
			 (this->GetLength(S2,E2,2) < this->minLen) )
			 this->DeleteEntry(i) ;
	}
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::SwapEntries(unsigned long i , unsigned long j) {

	Entry<GENOME_TYPE> tmp=this->entries[i];

	this->entries[i] = this->entries[j];
	this->entries[j] = tmp;
	tmp.Delete();
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::CleanDeleted() {

	Entry<GENOME_TYPE> *new_entries = new Entry<GENOME_TYPE>[numOfEntries] , *tmp;
	
	for( unsigned long i(0),j(0) ; i < this->tableSize ; i++) {
		if (this->entries[i].IsInUse()) 
			new_entries[j++] = this->entries[i] ;		
	}
			
	tmp = this->entries;
	this->entries = new_entries;
	this->tableSize = this->numOfEntries;
	delete[] tmp;
}	

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::PrintTable(ofstream& ofs,int org) {
	
	this->SortBy(org);

	ofs << "Org1: " << this->organism1.GetGenomeName() << "\t\t";
	ofs << "Org2: " << this->organism2.GetGenomeName() << endl;
	ofs << "START1	END1	LENGTH1	START2	END2	LENGTH2	ORIENTATION	IDENTIFIER" << endl;

	for ( unsigned long i=0 ; i < this->tableSize ; i++ ) {
		if ( !this->entries[i].IsInUse() ) continue;

		GENOME_TYPE s1,e1,s2,e2;
		string label;
		bool sign;
		this->entries[i].GetS1E1(s1,e1);
		this->entries[i].GetS2E2(s2,e2);
		label = this->entries[i].GetLabel();
		sign = this->entries[i].GetSign();

		ofs << s1 << "\t" << e1 << "\t" << this->GetLength(s1,e1,1) << "\t" << s2 << "\t" << e2 << "\t" << this->GetLength(s2,e2,2) << "\t" ;
		if ( sign ) ofs << "+"  ;
		else ofs << "-" ;
		ofs << "\t" << label << endl;

	}
}

//*************************************************************/

template<class GENOME_TYPE>
void Table<GENOME_TYPE>::CleanProphages(ifstream& fd ,int orgNum, double perc) {
	string st;
	GENOME_TYPE Pro_S,Pro_E,Table_S,Table_E;
	unsigned long i,tmp;
	double *perc_arr = new double[this->tableSize];
	bool flag=false;

	for( i=0 ; i < this->tableSize ; i++ )
		perc_arr[i] = 0;

	this->UpDateCircInfo();

	this->SortBy(orgNum);

	getline(fd, st);

	i=0;
	while ( (fd.peek() != EOF) && (i < this->tableSize) ) {
		fd >> Pro_S;		
		fd >> Pro_E;
		
		while ( (!this->entries[i].IsInUse()) && (i < this->tableSize) ) i++;
		if ( i >= this->tableSize ) break;

		// increse the "pointer" of the table
		i--;
		do {
			i++;

			while ( (!this->entries[i].IsInUse()) && (i < this->tableSize) ) i++;
			if ( i >= this->tableSize ) {
				flag = true;
				break;
			}
            
			if ( orgNum == 1 ) 
                this->entries[i].GetS1E1(Table_S,Table_E);
			else this->entries[i].GetS2E2(Table_S,Table_E);

		}while( (Intersec(i, Pro_S, Pro_E, orgNum) == 0) && (Pro_S > Table_S) ) ; /* to get to intersect entries */
		if ( flag ) break;

		// should increase the "pointer" of the prophages
		if ( (Intersec(i, Pro_S, Pro_E, orgNum) == 0) && (Pro_S < Table_S) ) continue;

		tmp = i;
		if ( orgNum == 1 ) 
            this->entries[tmp].GetS1E1(Table_S,Table_E);
        else 
			this->entries[tmp].GetS2E2(Table_S,Table_E);
		// check for all the entries the intersect with the prophage
		while( (Intersec(tmp,Pro_S,Pro_E,orgNum) != 0) || (Pro_E > Table_S)) {
			if ( Table_S < Table_E ) // entry isnt circular. circular entries are calculated later
                perc_arr[tmp] += Intersec(tmp,Pro_S,Pro_E,orgNum) ;
            
			tmp++;
			while ( (!this->entries[tmp].IsInUse()) && (tmp < this->tableSize) ) tmp++;
			if ( tmp >= this->tableSize ) {
				flag = true;
				break;
			}

			if ( orgNum == 1 ) 
                this->entries[tmp].GetS1E1(Table_S,Table_E);
            else this->entries[tmp].GetS2E2(Table_S,Table_E);
		} //  while
		if ( flag ) break;

		// Pro is circular, we must update of the entries of the begining
		if ( Pro_S > Pro_E )
		{
			tmp = 0;

			if ( orgNum == 1 ) 
                this->entries[tmp].GetS1E1(Table_S,Table_E);
            else this->entries[tmp].GetS2E2(Table_S,Table_E);

			while ( Pro_E > Table_S )
			{
				if ( Table_S < Table_E ) // entry isnt circular. circular entries are calculated later
                    perc_arr[tmp] += Intersec(tmp,Pro_S,Pro_E,orgNum) ;

                tmp++;

				if ( tmp >= this->tableSize ) break;
			
				if ( orgNum == 1 ) 
					this->entries[tmp].GetS1E1(Table_S,Table_E);
				else this->entries[tmp].GetS2E2(Table_S,Table_E);
			} // while
		} // if

		unsigned long firstCerc, lastCerc;
		if ( orgNum == 1 )
		{
			firstCerc = this->firstCircOrg1;
			lastCerc = this->lastCircOrg1;
		}
		else
		{
			firstCerc = this->firstCircOrg2;
			lastCerc = this->lastCircOrg2;
		}
		// check for circular entries
		for ( tmp = firstCerc ; tmp <= lastCerc ; tmp++ )
		{
			if ( orgNum == 1 ) 
                this->entries[tmp].GetS1E1(Table_S,Table_E);
            else 
				this->entries[tmp].GetS2E2(Table_S,Table_E);

			if ( Table_S < Table_E ) // entry isnt circular
				continue;

			perc_arr[tmp] += Intersec(tmp,Pro_S,Pro_E,orgNum) ;
		}
	
	} // while

	for( i=0 ; i < this->tableSize ; i++) 
		if ( perc_arr[i] >= perc ) this->DeleteEntry(i);

	delete perc_arr;	
}

//*************************************************************/

template<class GENOME_TYPE>
void Table<GENOME_TYPE>::CleanTE(ifstream& fd ,int orgNum, double perc) {
	string st;
	GENOME_TYPE TE_S=0,TE_E=0,Table_S=0,Table_E=0;
	unsigned long i,tmp;
	double *perc_arr = new double[this->tableSize];
	bool flag=false;

	for( i=0 ; i < this->tableSize ; i++ )
		perc_arr[i] = 0;

	this->UpDateCircInfo();

	this->SortBy(orgNum);

	getline(fd, st);	
	
	i=0;
	 
	while ( (fd.peek() != EOF) && (i < this->tableSize) ) { 		
		fd >> TE_S;
		fd >> TE_E;
		
		while ( (!this->entries[i].IsInUse()) && (i < this->tableSize) ) i++;
		if ( i >= this->tableSize ) break;

		// increase the "pointer" of the table
		i--;
		do {
			i++;

			while ( (!this->entries[i].IsInUse()) && (i < this->tableSize) ) i++;
			if ( i >= this->tableSize ) {
				flag = true;
				break;
			}
            
			if ( orgNum == 1 ) 
                this->entries[i].GetS1E1(Table_S,Table_E);
			else this->entries[i].GetS2E2(Table_S,Table_E);

		}while( (Intersec(i, TE_S, TE_E, orgNum) == 0) && (TE_S > Table_S) ) ; /* to get to intersect entries */
		if ( flag ) break;

		// should increase the "pointer" of the TEs
		if ( (Intersec(i, TE_S, TE_E, orgNum) == 0) && (TE_S < Table_S) ) continue;

		tmp = i;
		if ( orgNum == 1 ) 
            this->entries[tmp].GetS1E1(Table_S,Table_E);
        else 
			this->entries[tmp].GetS2E2(Table_S,Table_E);
		// check for all the entries the intersect with the TE
		while( (Intersec(tmp,TE_S,TE_E,orgNum) != 0) || (TE_E > Table_S) ) {
			if ( Table_S < Table_E ) // entry isnt circular. circular entries are calculated later
				perc_arr[tmp] += Intersec(tmp,TE_S,TE_E,orgNum) ;
            
			tmp++;
			while ( (!this->entries[tmp].IsInUse()) && (tmp < this->tableSize) ) tmp++;
			if ( tmp >= this->tableSize ) {
				flag = true;
				break;
			}

			if ( orgNum == 1 ) 
                this->entries[tmp].GetS1E1(Table_S,Table_E);
            else this->entries[tmp].GetS2E2(Table_S,Table_E);
		} //  while
		if ( flag ) break;

		// Pro is circular, we must update of the entries of the begining
		if ( TE_S > TE_E )
		{
			tmp = 0;

			if ( orgNum == 1 ) 
                this->entries[tmp].GetS1E1(Table_S,Table_E);
            else this->entries[tmp].GetS2E2(Table_S,Table_E);

			while ( (TE_E > Table_S) && (TE_S > Table_E) )
			{
				if ( Table_S < Table_E ) // entry isnt circular. circular entries are calculated later
                    perc_arr[tmp] += Intersec(tmp,TE_S,TE_E,orgNum) ;

                tmp++;

				if ( tmp >= this->tableSize ) break;
			
				if ( orgNum == 1 ) 
					this->entries[tmp].GetS1E1(Table_S,Table_E);
				else this->entries[tmp].GetS2E2(Table_S,Table_E);
			} // while
		} // if

		unsigned long firstCerc, lastCerc;
		if ( orgNum == 1 )
		{
			firstCerc = this->firstCircOrg1;
			lastCerc = this->lastCircOrg1;
		}
		else
		{
			firstCerc = this->firstCircOrg2;
			lastCerc = this->lastCircOrg2;
		}
		// check for circular entries
		for ( tmp = firstCerc ; tmp <= lastCerc ; tmp++ )
		{
			if ( orgNum == 1 ) 
                this->entries[tmp].GetS1E1(Table_S,Table_E);
            else 
				this->entries[tmp].GetS2E2(Table_S,Table_E);

			if ( Table_S < Table_E ) // entry isnt circular
				continue;

			perc_arr[tmp] += Intersec(tmp,TE_S,TE_E,orgNum) ;
		}
	
	} // while

	for( i=0 ; i < this->tableSize ; i++) 
		if ( perc_arr[i] >= perc ) this->DeleteEntry(i);

	delete perc_arr;
	
}

//*************************************************************/

template<class GENOME_TYPE>
GENOME_TYPE Table<GENOME_TYPE>::GetLength(GENOME_TYPE S, GENOME_TYPE E, int orgNum ) const{
	bool is_circular=false;
	GENOME_TYPE genum_len;


	if ( orgNum == 1 ) {
		genum_len = this->organism1.GetGenomeLen() ;
		if ( this->organism1.IsGenomeCircular() ) is_circular = true;
	}
	else {
		genum_len = this->organism2.GetGenomeLen() ;
		if ( this->organism2.IsGenomeCircular() ) is_circular = true;
	}
	
	if ( (!is_circular) || (S <= E) ) return E-S+1;
	else return genum_len-S+E+1;
}


//*************************************************************/

template<class GENOME_TYPE>
GENOME_TYPE Table<GENOME_TYPE>::InersecLen(GENOME_TYPE S, GENOME_TYPE E, GENOME_TYPE tmp_S,
										   GENOME_TYPE tmp_E ,int orgNum) const{

	// note : ( ) is for S,E  ,  [ ] is for tmp_S,tmp_E 

	//  ( [ ) ]    
	if ( (E >= tmp_S) && (E <= tmp_E) && (S <= tmp_S) ) {
		return GetLength(tmp_S,E,orgNum); 
	}
	//  [ ( ] )   
	if ( (E >= tmp_E) && (S <= tmp_E) && (S >= tmp_S) ) {
		return GetLength(S,tmp_E,orgNum); 
	}
	//  [ ( ) ]   
	if ( (S >= tmp_S) && (E <= tmp_E) ) {
		return GetLength(S,E,orgNum); 
	}
    //  ( [ ] ) 
	if ( (S <= tmp_S) && (E >= tmp_E) ) {
		return GetLength(tmp_S,tmp_E,orgNum);
	}

	// [ ] ( )  or  ( ) [ ]
	return 0;
}


//*************************************************************/

template<class GENOME_TYPE>
double Table<GENOME_TYPE>::Intersec(unsigned long entry, GENOME_TYPE S, GENOME_TYPE E,int orgNum/*, double perc*/) const{
	GENOME_TYPE tmp_S , tmp_E;
	bool is_cerc;

	if ( orgNum == 1 ) {
		is_cerc = this->organism1.IsGenomeCircular() ;
		this->entries[entry].GetS1E1(tmp_S,tmp_E);
	}
	else {
		is_cerc = this->organism2.IsGenomeCircular() ;
		this->entries[entry].GetS2E2(tmp_S,tmp_E);
	}

	// check if the genome isnt circular
	if ( !is_cerc ) {		
		return ((double)InersecLen(S,E,tmp_S,tmp_E,orgNum))/this->GetLength(tmp_S,tmp_E,orgNum);
	}
	else {  // genome is circular
		ManipulateStartsAndEnds(tmp_S,tmp_E,S,E,orgNum);
		return ((double)InersecLen(S,E,tmp_S,tmp_E,orgNum))/this->GetLength(tmp_S,tmp_E,orgNum);
	}

	return false;			
}

//*************************************************************/

template<class GENOME_TYPE>
void Table<GENOME_TYPE>::ManipulateStartsAndEnds(GENOME_TYPE& S1, GENOME_TYPE& E1, GENOME_TYPE& S2, GENOME_TYPE& E2,int orgNum) const{
	GENOME_TYPE genum_len;

	if ( orgNum == 1 )
		genum_len = this->organism1.GetGenomeLen();
	else genum_len = this->organism2.GetGenomeLen();

	// note : [ ] is for S1,E1  ,  ( ) is for S2,E2

	if ( (S1 < E1) && (S2 < E2) ) return ;
	
	if ( (S1 < E1) && (S2 > E2) ) {

		// [ ] ) (   or   [ ) ] (   
		if ( E1 < S2 ) {
            S1 += genum_len;
			E1 += genum_len;
			E2 += genum_len;
			return;
		}
		else {  //  ) ( [ ]   or   ) [ ( ]
			E2 += genum_len;
			return;
		}
	}

	if ( (S2 <= E2) && (S1 >= E1) ) {

		//  ( ] ) [  or   (  ) ] [  
		if ( E2 < S1 ) {
			E1 += genum_len;
			S2 += genum_len;
			E2 += genum_len;
			return;
		}

		//  ] ( [ )  or  ] [ ( )
		if ( E1 < S2 ) {
			E1 += genum_len;
			return;
		}
	}

	if ( (S1 > E1) && (S2 > E2) ) {
			E1 += genum_len;
			E2 += genum_len; 			
	}

}

//*************************************************************/

template <class GENOME_TYPE>
bool Table<GENOME_TYPE>::IsDuplication(unsigned long entry , unsigned of_entry , int orgNum ) const{
	GENOME_TYPE s,e,of_s,of_e;

	if ( orgNum == 1 ) {
		this->entries[entry].GetS1E1(s,e);
		this->entries[of_entry].GetS1E1(of_s,of_e);		
	}
	else {
		this->entries[entry].GetS2E2(s,e);
		this->entries[of_entry].GetS2E2(of_s,of_e);
	}
	
	if ( (this->Intersec(entry,of_s,of_e,orgNum) >= this->dupPerc ) || 
		    (this->Intersec(of_entry,s,e,orgNum) >= this->dupPerc) )
		return true;
	else return false;
}


//*************************************************************/
template <class GENOME_TYPE>
set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> > Table<GENOME_TYPE>::GetDupGroup(unsigned long entry , int orgNum , GENOME_TYPE *ends_arr){
	GENOME_TYPE S=0,E=0,tmp_S=0,tmp_E=0;
	unsigned long tmp=entry+1;
	set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> > s;

	s.insert(this->entries[entry]);

	if ( orgNum == 1 ) 
		this->entries[entry].GetS1E1(S,E);
	else 
		this->entries[entry].GetS2E2(S,E);

	do // for all the entries that come after 'entry' 
	{ 
		if (tmp >= this->tableSize) break;
		if ( !this->entries[tmp].IsInUse() ) {
			tmp++;
			continue;
		}

		if ( this->IsDuplication(tmp,entry,orgNum) )
			s.insert(this->entries[tmp]);

		tmp++; 
		if (tmp >= this->tableSize) break;
		
	}while( Intersec(tmp, S, E, orgNum) != 0 );

	if ( S > E ) // if 'entry' is circular, check entries from the begining
	{
		for ( tmp = 0 ; tmp < entry ; tmp++ )
		{
			if ( !this->entries[tmp].IsInUse() ) continue;

			if ( Intersec(tmp, S, E, orgNum) == 0 )
				break;

			if ( this->IsDuplication(tmp,entry,orgNum) )
				s.insert(this->entries[tmp]);
		} // for
	} // if

	unsigned long firstCerc, lastCerc;

	if ( orgNum == 1 )
	{
		firstCerc = this->firstCircOrg1;
		lastCerc = this->lastCircOrg1;
	}
	else
	{
		firstCerc = this->firstCircOrg2;
		lastCerc = this->lastCircOrg2;
	}

	// check for the the entries that are circular
	for ( tmp = firstCerc ; tmp <= lastCerc ; tmp++ )
	{
		if ( !this->entries[tmp].IsInUse() ) continue;

		if ( orgNum == 1 )
            this->entries[tmp].GetS1E1(tmp_S,tmp_E);
		else
			this->entries[tmp].GetS2E2(tmp_S,tmp_E);

		if ( tmp_S < tmp_E ) // not circular
			continue;

		if ( this->IsDuplication(tmp,entry,orgNum) )
				s.insert(this->entries[tmp]);
	}

	if ( entry == 0 )
		return s;

	tmp = entry-1;

	do  // for all entries that come before 'entry'
	{
		if ( tmp+1 == 0 ) break;
		if ( !this->entries[tmp].IsInUse() ) {
			if ( tmp == 0 )
				return s;
			tmp--;
			continue;
		}

		if ( this->IsDuplication(tmp,entry,orgNum) )
			s.insert(this->entries[tmp]);

		if ( tmp == 0 )
			return s;
	}while(ends_arr[tmp--] >= S);	

	return s;
}

//*************************************************************/

template<class GENOME_TYPE>
void Table<GENOME_TYPE>::Join(unsigned long *arr) {
	
	for (unsigned long i=0 ; i < this->tableSize ; i++) {
		if (!this->entries[i].IsInUse()) continue;

		unsigned long entry = arr[i];
		GENOME_TYPE iS1,iE1,iS2,iE2;
		this->entries[i].GetS1E1(iS1,iE1);
		this->entries[i].GetS2E2(iS2,iE2);
		bool sign = this->entries[i].GetSign();
		string st;
		bool flag = true;
		
		for (unsigned long j=0 ; j < this->tableSize ; j++) {

			if (!this->entries[j].IsInUse()) continue;
			if (arr[j] != entry) continue;

			GENOME_TYPE tmpS1,tmpE1,tmpS2,tmpE2;
			this->entries[j].GetS1E1(tmpS1,tmpE1);
			this->entries[j].GetS2E2(tmpS2,tmpE2);

			if ( iS1 > iE1 ) // entry i is circular in org 1
			{
				if ( tmpS1 > tmpE1 ) // entry j is circular in org 1
				{
					if ( tmpS1 < iS1 ) iS1 = tmpS1;
					if ( iE1 < tmpE1 ) iE1 = tmpE1 ;
				}
				else
				{
					//check whether entry j is closer to the start of i or the end of i
					if ( abs((long)(iS1-tmpE1)) < abs((long)(tmpS1-iE1)) )
						iS1 = tmpS1;
					else
						iE1 = tmpE1;
				}

			}
			else  // entry i is not circular in org 1
			{
				if ( tmpS1 < tmpE1 ) // entry j is not circular in org 1
				{
					if ( !this->organism1.IsGenomeCircular() )
					{
                        if ( tmpS1 < iS1 ) iS1 = tmpS1;
                        if ( iE1 < tmpE1 ) iE1 = tmpE1 ;
					}
					else  //  should chek for this situation:    .  ( )				[ ] .
					{	  //  then the joined entries should be: .    }             {   .
						if ( abs((long)(this->organism1.GetGenomeLen()-tmpE1+iS1)) < abs((long)(tmpS1-iE1)) )
							iS1 = tmpS1;
						else
						{
							if ( tmpS1 < iS1 ) iS1 = tmpS1;
							if ( iE1 < tmpE1 ) iE1 = tmpE1 ;
						}
					}
				}
				else // entry j is circular in org 1
				{
					//check whether entry i is closer to the start of j or the end of j
					if ( abs((long)(tmpS1-iE1)) > abs((long)(iS1-tmpE1)) )
						iS1 = tmpS1;
					else
						iE1 = tmpE1 ;
				}
			}

			if ( iS2 > iE2 ) // entry i is circular in org 2
			{
				if ( tmpS2 > tmpE2 ) // entry j is circular in org 2
				{
					if ( tmpS2 < iS2 ) iS2 = tmpS2;
					if ( iE2 < tmpE2 ) iE2 = tmpE2 ;
				}
				else
				{
					//check whether entry j is closer to the start of i or the end of i
					if ( abs((long)(iS2-tmpE2)) < abs((long)(tmpS2-iE2)) )
						iS2 = tmpS2;
					else
						iE2 = tmpE2;
				}

			}
			else  // entry i is not circular in org 2
			{
				if ( tmpS2 < tmpE2 ) // entry j is not circular in org 2
				{
					if ( !this->organism2.IsGenomeCircular() )
					{
                        if ( tmpS2 < iS2 ) iS2 = tmpS2;
                        if ( iE2 < tmpE2 ) iE2 = tmpE2 ;
					}
					else  //  should chek for this situation:    .  ( )				[ ] .
					{	  //  then the joined entries should be: .    }             {   .
						if ( abs((long)(this->organism2.GetGenomeLen()-tmpE2+iS2)) < abs((long)(tmpS2-iE2)) )
							iS2 = tmpS2;
						else
						{
							if ( tmpS2 < iS2 ) iS2 = tmpS2;
							if ( iE2 < tmpE2 ) iE2 = tmpE2 ;
						}
					}
				}
				else // entry j is circular in org 2
				{
					//check whether entry i is closer to the start of j or the end of j
					if ( abs((long)(tmpS2-iE2)) > abs((long)(iS2-tmpE2)) )
						iS2 = tmpS2;
					else
						iE2 = tmpE2 ;
				}
			}

			if (!flag) st += ",";
			else flag=false;

			st.append(this->entries[j].GetLabel());			

			this->DeleteEntry(j);
		}
		this->entries[i].InseretEntry(iS1,iE1,iS2,iE2,sign,st);
		this->numOfEntries++;
	}
}

//*************************************************************/

template<class GENOME_TYPE>	
void Table<GENOME_TYPE>::Clustering(){

    Table<GENOME_TYPE> *t2 = new Table(*this);
	unsigned long i=0, j=0, i_=0, j_=0;
	unsigned long *consecutive_arr = new unsigned long[this->tableSize]; 
	for(i=0; i<this->tableSize ; i++)
		consecutive_arr[i] = i;

	t2->UpDateCircInfo();
	this->UpDateCircInfo();

	this->SortBy(1);
	t2->SortBy(2);
	

	GENOME_TYPE *arr1= new GENOME_TYPE[this->tableSize] , *arr2 = new GENOME_TYPE[this->tableSize];

	this->FillEnds(arr1,1);
	t2->FillEnds(arr2,2);

	for(i=0 ; i < this->tableSize ; i++) {

		bool circSearchFlag = false;

		if (!this->entries[i].IsInUse() ) continue;

		j=i+1;
		
		while ( (j < this->tableSize) && ( !this->entries[j].IsInUse() || this->IsDuplication(j,i,1)) )
			j++;

		if ( !this->organism1.IsGenomeCircular() )
		{
			if (j >= this->tableSize) continue;
		}
		else
		{
			// if j reached the end of the table, we should search from the begining
			if (j >= this->tableSize) 
			{
				circSearchFlag = true;
				j = 0;
				while ( (j < i+1) && ( !this->entries[j].IsInUse() || this->IsDuplication(j,i,1)) )
					j++;
				if ( j == i+1 )
					continue;
			}
		}

		GENOME_TYPE S1,E1,S2,E2;
		this->entries[i].GetS1E1(S1,E1);
		this->entries[i].GetS2E2(S2,E2);
		i_=t2->BinarySearch(S1,E1,S2,E2,2);

		int sign;

		if ( t2->entries[i_].GetSign() ) sign = 1;
		else sign = -1;

		j_ = i_ + sign;

		while ( (j_ < this->tableSize) && (j_ >= 0) 
			    && ( !t2->entries[j_].IsInUse() || t2->IsDuplication(j_,i_,2)) )
				j_ += sign;
		
		if ( !t2->organism2.IsGenomeCircular() )
		{
            if ( (j_ >= t2->tableSize) || (j_ < 0) ) continue;
		}
		else
		{
			if ( j_ >= this->tableSize )  // || (j_ < 0) )
			{
				circSearchFlag = true;
				j_ = 0;
				while ( (j_ < i_+sign) && ( !this->entries[j_].IsInUse() || this->IsDuplication(j_,i_,2)) )
					j_ += sign;
				if ( j_ == i_+sign )
					continue;
			}
			if ( j_ < 0 )
			{
				j_ = this->tableSize-1;
				while ( (j_ > i_+sign) && ( !t2->entries[j_].IsInUse() || t2->IsDuplication(j_,i_,2)) )
					j_ += sign;
				if ( j_ == i_+sign )
					continue;
			}
		}		

		set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> > s1,s2,intersec;
		s1 = this->GetDupGroup(j,1,arr1);
		s2 = t2->GetDupGroup(j_,2,arr2);

		this->FilterSet(s1,i,1,false);
		t2->FilterSet(s2,i_,2,true,t2->entries[i_].GetSign());

		insert_iterator<set<Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> > > iter(intersec,intersec.begin());
		set_intersection(s1.begin(),s1.end(),s2.begin(), s2.end(),iter,compare_entries_1<GENOME_TYPE>() );

		if ( intersec.size() == 1 ) {
			(*(intersec.begin())).GetS1E1(S1,E1);
			(*(intersec.begin())).GetS2E2(S2,E2);
			j = this->BinarySearch(S1,E1,S2,E2,1);
			if ( (consecutive_arr[j] == j) && !circSearchFlag )
                consecutive_arr[j] = consecutive_arr[i];
			else if ( (consecutive_arr[j] == j) && circSearchFlag )
			{
				consecutive_arr[i] = consecutive_arr[j];
			}
			else 
				consecutive_arr[j] = j;			
		}		
	}

	this->Join(consecutive_arr);
	delete arr1;
	delete arr2;
	delete consecutive_arr;
	delete t2;
	
}

//*************************************************************/

template<class GENOME_TYPE>
void Table<GENOME_TYPE>::FillEnds(GENOME_TYPE *arr,int orgNum) const{
	GENOME_TYPE max=0,S,E;

	for (unsigned long i=0; i< this->tableSize ; i++) {
		if ( orgNum == 1 ) 
			this->entries[i].GetS1E1(S,E);
		else 
			this->entries[i].GetS2E2(S,E);

		if ( E > max ) {
			arr[i] = E;
			max = E;
		}
		else arr[i] = max;
	}
}

//*************************************************************/

template<class GENOME_TYPE>
unsigned long Table<GENOME_TYPE>::BinarySearch(GENOME_TYPE S1,GENOME_TYPE E1,GENOME_TYPE S2,GENOME_TYPE E2,int orgNum) const{
	unsigned long begin,mid,end;
	GENOME_TYPE tmp_S1,tmp_E1,tmp_S2,tmp_E2;
	begin=0;
	end=this->tableSize;

	while ( begin <= end) {
        mid = (begin+end)/2;

		this->entries[mid].GetS1E1(tmp_S1,tmp_E1);
		this->entries[mid].GetS2E2(tmp_S2,tmp_E2);

		if ( (tmp_S1 == S1) && (tmp_E1 == E1) && (tmp_S2 == S2) && (tmp_E2 == E2) ) 
			return mid;

		if ( orgNum == 1 ) {
			if ( tmp_S1 < S1 ) 
				begin = mid+1;
			else if ( tmp_S1 > S1 )
				end = mid-1;
			else if ( tmp_E1 < E1 ) // S1 == tmp_S1
				begin = mid+1;
			else if ( tmp_E1 > E1 )
				end = mid-1;
			else if ( tmp_S2 < S2 ) // S1 == tmp_S1 and E1 == tmp_E1
				begin = mid+1;
			else if ( tmp_S2 > S2 )
				end = mid-1;
			else if ( tmp_E2 < E2 ) // // S1 == tmp_S1 and E1 == tmp_E1 and S2 == tmp_S2
				begin = mid+1;
			else end = mid-1;
		}
		else {
			if ( tmp_S2 < S2 ) 
				begin = mid+1;
			else if ( tmp_S2 > S2 )
				end = mid-1;
			else if ( tmp_E2 < E2 ) // S2 == tmp_S2
				begin = mid+1;
			else if ( tmp_E2 > E2 )
				end = mid-1;
			else if ( tmp_S1 < S1 ) // S2 == tmp_S2 and E2 == tmp_E2
				begin = mid+1;
			else if ( tmp_S1 > S1 )
				end = mid-1;
			else if ( tmp_E1 < E1 ) // // S2 == tmp_S2 and E2 == tmp_E2 and S1 == tmp_S1
				begin = mid+1;
			else end = mid-1;
		}
	}
	return 0; 
}

//*************************************************************/


template <class GENOME_TYPE>
void Table<GENOME_TYPE>::FilterSet(set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> >& s,
								   unsigned long j, int org, bool flag, bool sign) {

	entryIter i = s.begin() , tmp;

	if ( s.empty() ) return;

	while ( i != s.end() ) {
		if ( flag ) {
            if ( (*i).GetSign() != sign ) {
				tmp = i ;
				i.operator++();
				s.erase(tmp);
				if ( i == s.end() ) return;
				continue;
			}
		}
		
		GENOME_TYPE S,E,S_,E_;
		if ( org == 1 ) {
            (*i).GetS1E1(S,E);
			this->entries[j].GetS1E1(S_,E_);
		}
		else {
			(*i).GetS2E2(S,E);
			this->entries[j].GetS2E2(S_,E_);
		}

		double intersec_perc = this->Intersec(j,S,E,org);
		if ( ((intersec_perc*(E_-S_+1)) >= 0.5*(E-S)) || ((intersec_perc*(E_-S_+1)) >= 0.5*(E_-S_)) ) {
			tmp = i ;
			i.operator++();
			s.erase(tmp);
			if ( i == s.end() ) return;
			i.operator--();
		}		

        i.operator++();
	}
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::RemoveOverlaps() {

	GENOME_TYPE jS1,jE1,jS2,jE2;
	unsigned long i,j;
	unsigned long *arr = new unsigned long[this->tableSize];
	for ( i=0 ; i <this->tableSize ; i++ )
		arr[i] = i;

	this->SortBy(1);

	for ( i=0 , j=0 ; i <this->tableSize ; i++ ) {
		if ( !this->entries[i].IsInUse() ) continue;
		GENOME_TYPE iS1,iE1,iS2,iE2;
		this->entries[i].GetS1E1(iS1,iE1);
		this->entries[i].GetS2E2(iS2,iE2);

		bool deleted = false;

		j = i;
		// check for all the entries the come after i
		do {
			j++;
			if ( j >= this->tableSize ) break;

			this->entries[j].GetS1E1(jS1,jE1);
			this->entries[j].GetS2E2(jS2,jE2);

			if ( !this->entries[j].IsInUse() ) continue;			

			if ( JoinForOverlap(i, j) )
			{
                deleted = true;
				break;
			}								
		} while( this->Intersec(i,jS1,jE1,1) != 0 );
		if (deleted) 
			continue;
		
		// if entry i is circular in one organism at least, then entries from the begining of the table
		// should be checked
		if ( (iS1 > iE1)  )  
        { 
			for ( j = 0 ; j < this->tableSize ; j++ )
			{
				if ( !this->entries[j].IsInUse() ) continue;

				this->entries[j].GetS1E1(jS1,jE1);
				this->entries[j].GetS2E2(jS2,jE2);

				// if there is no intersection, then for all entries after j there is no intersection
				if ( this->Intersec(i,jS1,jE1,1) == 0 ) break;

				if ( JoinForOverlap(i, j) )
					break;
			} // for
		} // if
	} // for
}


//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::RemoveNuisances(double orthPerc) {
	Table<GENOME_TYPE> *t2 = new Table(*this);
	unsigned long i=0, i_=0;
	bool break1=false,break2=false; /// bad names

	t2->UpDateCircInfo();
	this->SortBy(1);
	t2->SortBy(2);
	

	GENOME_TYPE *arr1= new GENOME_TYPE[this->tableSize] , *arr2 = new GENOME_TYPE[this->tableSize];
	int *entrysToDelete= new int [this->tableSize];
	if ( entrysToDelete == NULL )
		cout << "null" << endl;

	for ( i=0 ; i<this->tableSize ; i++ )
		entrysToDelete[i] = 0;
				
	
	this->FillEnds(arr1,1);	
	t2->FillEnds(arr2,2);	

	for ( i=0 ; i<this->tableSize ; i++ ) {
		

		break1=false;
		break2=false;

		GENOME_TYPE iS1,iE1,iS2,iE2,i_S2,i_E2;
		this->entries[i].GetS1E1(iS1,iE1);
		this->entries[i].GetS2E2(iS2,iE2);		

		i_=t2->BinarySearch(iS1,iE1,iS2,iE2,2);
		t2->entries[i_].GetS2E2(i_S2,i_E2);		

		set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> > s1,s2;
		s1 = this->GetDupGroup(i,1,arr1);		
		s2 = t2->GetDupGroup(i_,2,arr2);		

		entryIter itr1 = s1.begin() , itr2 = s2.begin();
		// scan the set
		while ( itr1 != s1.end() ) 
		{
			GENOME_TYPE itr1S1,itr1E1;
			(*itr1).GetS1E1(itr1S1,itr1E1);
			if ( ((double)this->GetLength(iS1,iE1,1))/this->GetLength(itr1S1,itr1E1,1) <= orthPerc ) {
				break1 = true;
				break;
			}
			itr1++;
		}
		// scan the set
		while ( itr2 != s2.end() ) 
		{
			GENOME_TYPE itr2S2,itr2E2;
			(*itr2).GetS2E2(itr2S2,itr2E2);
			if ( ((double)t2->GetLength(i_S2,i_E2,2))/t2->GetLength(itr2S2,itr2E2,2) <= orthPerc ) {
				break2 = true;
				break;
			}
			itr2++;
		}
		
		if ( break1 && break2 )
		{
			entrysToDelete[i] = 1;			
		}
	}

	
	for ( i=0 ; i<this->tableSize ; i++ )
		if ( entrysToDelete[i] )
			this->DeleteEntry(i);	

	delete t2;
	delete arr1;
	delete arr2;
	delete entrysToDelete;

}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::RemoveParalogs(double orthPerc) {
	Table<GENOME_TYPE> *t2 = new Table(*this);
	unsigned long i=0, i_=0;
	
	t2->UpDateCircInfo();
	this->SortBy(1);
	t2->SortBy(2);
	

	GENOME_TYPE *arr1= new GENOME_TYPE[this->tableSize] , *arr2 = new GENOME_TYPE[this->tableSize];
	unsigned long *entrysToDelete= new unsigned long[this->tableSize];
	

	for ( i=0 ; i<this->tableSize ; i++ )
		entrysToDelete[i] = 0;
				

	this->FillEnds(arr1,1);
	t2->FillEnds(arr2,2);

	for ( i=0 ; i<this->tableSize ; i++ ) {

		if ( !this->entries[i].IsInUse() ) continue;
		
		GENOME_TYPE iS1,iE1,iS2,iE2,i_S2,i_E2;
		this->entries[i].GetS1E1(iS1,iE1);
		this->entries[i].GetS2E2(iS2,iE2);

		i_=t2->BinarySearch(iS1,iE1,iS2,iE2,2);
		t2->entries[i_].GetS2E2(i_S2,i_E2);
		

		set< Entry<GENOME_TYPE> , compare_entries_1<GENOME_TYPE> > s1,s2;
		s1 = this->GetDupGroup(i,1,arr1);
		s2 = t2->GetDupGroup(i_,2,arr2);

		
		entryIter itr1 = s1.begin() , itr2 = s2.begin();

		bool break1=false,break2=false; 
		// scan the set
		while ( itr1 != s1.end() ) 
		{
			GENOME_TYPE itr1S1,itr1E1;
			(*itr1).GetS1E1(itr1S1,itr1E1);
			if ( ((double)this->GetLength(iS1,iE1,1))/this->GetLength(itr1S1,itr1E1,1) <= orthPerc ) {
				break1 = true;
#if PRINT_DELETED_PARALOGS
					this->deletedParalog1[this->deletedParalogCount1++] = this->entries[i];
#endif
				break;
			}
			itr1++;
		}
		// scan the set
		while ( itr2 != s2.end() ) 
		{
			GENOME_TYPE itr2S2,itr2E2;
			(*itr2).GetS2E2(itr2S2,itr2E2);
			if ( ((double)t2->GetLength(i_S2,i_E2,2))/t2->GetLength(itr2S2,itr2E2,2) <= orthPerc ) {
				break2 = true;
#if PRINT_DELETED_PARALOGS
					this->deletedParalog2[this->deletedParalogCount2++] = this->entries[i];
#endif
				break;
			}
			itr2++;
		}
		if ( break1 || break2 )
		{
			entrysToDelete[i] = 1;			
		}
	}



	for ( i=0 ; i<this->tableSize ; i++ )
		if ( entrysToDelete[i] )
			this->DeleteEntry(i);

	delete t2;
	delete arr1;
	delete arr2;
	delete entrysToDelete;	
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::IterativeRemoveOverlaps()
{
	this->CleanDeleted();
	int lastSize=0 , currSize=this->numOfEntries;

	while ( lastSize != currSize )
	{
		this->RemoveOverlaps();
		lastSize = currSize;
		currSize = this->numOfEntries;
	}
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::Loop(double orthPerc,const char* deletedParFile1,const char* deletedParFile2)
{
	int lastSize=0 , currSize=this->numOfEntries;

#if PRINT_DELETED_PARALOGS
	this->deletedParalog1 = new Entry<GENOME_TYPE>[this->tableSize];
	this->deletedParalog2 = new Entry<GENOME_TYPE>[this->tableSize];
	this->deletedParalogCount1 = this->deletedParalogCount2 = 0;
#endif

	while ( lastSize != currSize )
	{
		this->CleanDeleted();
		
		this->Clustering();

		this->CleanDeleted();
		this->UpDateCircInfo();

		this->RemoveNuisances(orthPerc);

		this->CleanDeleted();
		this->UpDateCircInfo();

		this->RemoveParalogs(orthPerc);

		lastSize = currSize;
		currSize = this->numOfEntries;		
	}
	
#if PRINT_DELETED_PARALOGS
	ofstream fd1, fd2;

	fd1.open(deletedParFile1);
	fd2.open(deletedParFile2);
	if ( fd1.is_open() && fd2.is_open() )
	{
		this->PrintDeletedParalogs(fd1,fd2);
		fd1.close();
		fd2.close();		
	}		
#endif
}


//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::RemoveFinalConflicts()
{
	this->CleanDeleted();

	GENOME_TYPE iS1, iE1, jS1, jE1, iS2, iE2, jS2, jE2;
	unsigned long i, j, *entriesToDelete = new unsigned long[this->tableSize], 
						*entriesToDeleteOrg2 = new unsigned long[this->tableSize];

	for ( i=0 ; i<this->tableSize ; i++)
		entriesToDelete[i] = entriesToDeleteOrg2[i] = 0;


	this->SortBy(1);

	for ( i=0 ; i<this->tableSize ; i++)
	{
		this->entries[i].GetS1E1(iS1,iE1);
		
		j = i;
		// check for the entries that come after entry i
		do {
			j++;
			if ( j >= this->tableSize ) break;

			this->entries[j].GetS1E1(jS1,jE1);

			if ( !this->entries[j].IsInUse() ) continue;			
			
			if ( this->IsDuplication(i,j,1) )
			{
				entriesToDelete[i] = entriesToDelete[j] = 1;
			}
		} while( this->Intersec(i,jS1,jE1,1) != 0 );	

		if ( iS1 > iE1 ) // entry i is circular, must check for entries from the beginning
		{
			for ( j=0 ; j < i ; j++ )
			{
				if ( !this->entries[j].IsInUse() ) continue;

				this->entries[j].GetS1E1(jS1,jE1);
                if ( this->Intersec(i,jS1,jE1,1) == 0 )
					break;

				if ( this->IsDuplication(i,j,1) )
					entriesToDelete[i] = entriesToDelete[j] = 1;
			} // for
		} // if

	} // for

	
	this->SortBy(2);

	for ( i=0 ; i<this->tableSize ; i++)
	{
		this->entries[i].GetS1E1(iS1,iE1);
		this->entries[i].GetS2E2(iS2,iE2);

		j = i;
		// check for the entries that come after entry i
		do {
			j++;
			if ( j >= this->tableSize ) break;

			this->entries[j].GetS1E1(jS1,jE1);
			this->entries[j].GetS2E2(jS2,jE2);

			if ( !this->entries[j].IsInUse() ) continue;			

			if ( this->IsDuplication(i,j,2) )
			{
				entriesToDeleteOrg2[i] = entriesToDeleteOrg2[j] = 1;
			}
		} while( this->Intersec(i,jS2,jE2,2) != 0 );	

		if ( iS2 > iE2 ) // entry i is circular, must check for entries from the beginning
		{
			for ( j=0 ; j < i ; j++ )
			{
				if ( !this->entries[j].IsInUse() ) continue;

				this->entries[j].GetS2E2(jS2,jE2);
                if ( this->Intersec(i,jS2,jE2,2) == 0 )
					break;

				if ( this->IsDuplication(i,j,2) )
					entriesToDelete[i] = entriesToDelete[j] = 1;
			} // for
		} // if
	
	} // for

	for ( i=0 ; i<this->tableSize ; i++)
		if ( entriesToDeleteOrg2[i] )
			this->DeleteEntry(i);

	this->SortBy(1);

	for ( i=0 ; i<this->tableSize ; i++)
		if ( entriesToDelete[i] )
			this->DeleteEntry(i);

	delete entriesToDelete;
	delete entriesToDeleteOrg2;
}


//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::RemoveFinalOverlaping()
{
	GENOME_TYPE iS1, iE1, iS2, iE2, jS1, jE1, jS2, jE2;
	unsigned long i, j;

	this->SortBy(1);

	for ( i=0 ; i<this->tableSize ; i++)
	{
		if ( !this->entries[i].IsInUse() ) continue;

		this->entries[i].GetS1E1(iS1,iE1);
		
		j = i;
		// check for the entries that come after entry i
		do {
			j++;
			if ( (j >= this->tableSize) || (j == i) ) break;

			this->entries[j].GetS1E1(jS1,jE1);
			this->entries[j].GetS2E2(jS2,jE2);

			if ( !this->entries[j].IsInUse() ) continue;			

			double intersec_perc = this->Intersec(i,jS1,jE1,1);
			if ( intersec_perc > 0 )
			{
				GENOME_TYPE intersecLen = (GENOME_TYPE)(intersec_perc * this->GetLength(iS1,iE1,1)) ;
				this->entries[j].SetS1E1((jS1+intersecLen)%this->organism1.GetGenomeLen(),jE1);
				if ( this->entries[j].GetSign() )
				{					
					this->entries[j].SetS2E2((jS2 + (unsigned long)(((double)intersecLen)*this->GetLength(jS2,jE2,2)/this->GetLength(jS1,jE1,1) ))%this->organism2.GetGenomeLen(), jE2);
				}
				else
					this->entries[j].SetS2E2(jS2,((unsigned long)( jE2 -  ((double)intersecLen)*this->GetLength(jS2,jE2,2)/this->GetLength(jS1,jE1,1)))%this->organism2.GetGenomeLen() );
			}
			// if entry i is circular and we finished scanning all entries after i,
			// we should scan entries from the begining
			if ( (j == this->tableSize-1) && (iS1 > iE1) )
				j = -1;			
		} while( this->Intersec(i,jS1,jE1,1) != 0 );
	}

	this->SortBy(2);

	for ( i=0 ; i<this->tableSize ; i++)
	{
		if ( !this->entries[i].IsInUse() ) continue;

		this->entries[i].GetS2E2(iS2,iE2);

		j = i;
		// check for the entries that come after entry i
		do {
			j++;
			if ( (j >= this->tableSize) || (j == i) ) break;

			this->entries[j].GetS1E1(jS1,jE1);
			this->entries[j].GetS2E2(jS2,jE2);

			if ( !this->entries[j].IsInUse() ) continue;			

			double intersec_perc = this->Intersec(i,jS2,jE2,2);
			if ( intersec_perc > 0 )
			{
				GENOME_TYPE intersecLen = (GENOME_TYPE)(intersec_perc * this->GetLength(iS2,iE2,2)) ;
				this->entries[j].SetS2E2((jS2+intersecLen)%this->organism2.GetGenomeLen(),jE2);
				if ( this->entries[j].GetSign() )
					this->entries[j].SetS1E1(((unsigned long)(jS1 + ((double)intersecLen)*this->GetLength(jS1,jE1,1)/this->GetLength(jS2,jE2,2) ))%this->organism1.GetGenomeLen(), jE1);
				else
					this->entries[j].SetS1E1(jS1,((unsigned long)( jE1 -  + ((double)intersecLen)*this->GetLength(jS1,jE1,1)/this->GetLength(jS2,jE2,2)))%this->organism1.GetGenomeLen() );
			}
			// if entry i is circular and we finished scanning all entries after i,
			// we should scan entries from the begining
			if ( (j == this->tableSize-1) && (iS2 > iE2) )
				j = -1;
			} while( this->Intersec(i,jS2,jE2,2) != 0 );
	}
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::UpDateCircInfo()
{
	GENOME_TYPE S1, E1, S2, E2;
	unsigned long i;

	this->firstCircOrg1 = this->firstCircOrg2 = this->tableSize;
	this->lastCircOrg1 = this->lastCircOrg2 = 0;

	if ( this->organism1.IsGenomeCircular() )
	{
		this->SortBy(1);

		for (i=0 ; i<this->tableSize ; i++)
		{
			this->entries[i].GetS1E1(S1,E1);
			
			if ( S1 > E1 ) // entry is circular
			{
				if ( firstCircOrg1 == this->tableSize )
					firstCircOrg1 = i;

				lastCircOrg1 = i;
			}
		}
	}
	
	if ( this->organism2.IsGenomeCircular() )
	{
		this->SortBy(2);

		for (i=0 ; i<this->tableSize ; i++)
		{
			this->entries[i].GetS2E2(S2,E2);
			
			if ( S2 > E2 ) // entry is circular
			{
				if ( firstCircOrg2 == this->tableSize )
					firstCircOrg2 = i;

				lastCircOrg2 = i;
			}
		}
	}
}

//*************************************************************/

template <class GENOME_TYPE>
bool Table<GENOME_TYPE>::JoinForOverlap(unsigned long i, unsigned long j)
{
	GENOME_TYPE iS1, iE1, iS2, iE2, jS1, jE1, jS2, jE2 ; 

	this->entries[i].GetS1E1(iS1,iE1);
	this->entries[i].GetS2E2(iS2,iE2);

	this->entries[j].GetS1E1(jS1,jE1);
	this->entries[j].GetS2E2(jS2,jE2);


    if ( (this->Intersec(i,jS1,jE1,1) > 0) && (this->Intersec(i,jS2,jE2,2) > 0) 
				&& (this->entries[i].GetSign() == this->entries[j].GetSign()) ) 
	{

		string st = this->entries[i].GetLabel() ;

		if ( iS1 > iE1 ) // entry i is circular in org 1
		{
			if ( jS1 > jE1 ) // entry j is circular in org 1
			{
				if ( iS1 < jS1 ) jS1 = iS1;
				if ( iE1 > jE1 ) jE1 = iE1;
			}
			else // entry j is not circular in org 1
			{
				if ( jE1 >= iS1 ) //  . )		[ ( ] .
					jE1 = iE1 ;
				else			 //  . [ ) ]	    ( .
					jS1 = iS1;
			}
		}
		else  // entry i is not circular in org 1
		{
			if ( jS1 < jE1 ) // entry j is not circular in org 1
			{
				if ( iS1 < jS1 ) jS1 = iS1;
				if ( iE1 > jE1 ) jE1 = iE1;
			}
			else // entry j is circular in org 1
			{
				if ( iE1 >= jS1 ) //  . ]		( [ ) .
					jS1 = iS1 ;
				else			 //  . ( ] )	    [ .
					jE1 = iE1;
			}
		}

		if ( iS2 > iE2 ) // entry i is circular in org 2
		{
			if ( jS2 > jE2 ) // entry j is circular in org 2
			{
				if ( iS2 < jS2 ) jS2 = iS2;
				if ( iE2 > jE2 ) jE2 = iE2;
			}
			else // entry j is not circular in org 2
			{
				if ( jE2 >= iS2 ) //  . )		[ ( ] .
					jE2 = iE2 ;
				else			 //  . [ ) ]	    ( .
					jS2 = iS2;
			}
		}
		else  // entry i is not circular in org 2
		{
			if ( jS2 < jE2 ) // entry j is not circular in org 2
			{
				if ( iS2 < jS2 ) jS2 = iS2;
				if ( iE2 > jE2 ) jE2 = iE2;
			}
			else // entry j is circular in org 2
			{
				if ( iE2 >= jS2 ) //  . ]		( [ ) .
					jS2 = iS2 ;
				else			 //  . ( ] )	    [ .
					jE2 = iE2;
			}
		}

		st += ";";
		st.append(this->entries[j].GetLabel());			

		this->entries[j].InseretEntry(jS1,jE1,jS2,jE2,this->entries[j].GetSign(),st);
		
		this->DeleteEntry(i);

		return true;
	} // if

	return false;
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::ReadFromProbFile(ifstream& fd, string &genumName, GENOME_TYPE &genumLen, bool &circ)
{
	string tmp;
	char circChar, c;

	// dont need first line
	getline(fd, tmp);
	
	genumName = "";
	
	fd.read(&c, sizeof(char));
	while ( c != '\t' )
	{
		genumName += c;
		fd.read(&c, sizeof(char)); 
	}
	
	fd >> genumLen;

	fd >> tmp; fd >> tmp;

	fd >> circChar;

	if ( circChar == 'T' )
		circ = true;
	else 
		circ = false;

	fd.close();
}

//*************************************************************/

template <class GENOME_TYPE>
void Table<GENOME_TYPE>::PrintDeletedParalogs(ofstream& file1, ofstream& file2)
{
	Table<GENOME_TYPE> *t1, *t2;
	unsigned long  i, j;

	t1 = new Table<GENOME_TYPE>();
	t2 = new Table<GENOME_TYPE>();

	t1->entries = this->deletedParalog1;
	t2->entries = this->deletedParalog2;

	t1->tableSize = t2->tableSize = this->tableSize ;

	t1->organism1 = this->organism1;
	t1->organism2 = this->organism2;

	t2->organism1 = this->organism1;
	t2->organism2 = this->organism2;

	t1->numOfEntries = this->deletedParalogCount1;
	t2->numOfEntries = this->deletedParalogCount2;

	t1->CleanDeleted();
	t2->CleanDeleted();

	t1->SortBy(1);
	t2->SortBy(2);

	GENOME_TYPE jS1, jE1, jS2, jE2, iS1, iE1, iS2, iE2 ;

	for ( i=0 ; i<t1->tableSize ; i++ )
	{
		t1->entries[i].GetS1E1(iS1,iE1);
		
		j = i;
		// check for all the entries the come after i
		do {
			j++;
			if ( (j >= t1->tableSize) && (iS1 > iE1) )
			{
				j = -1;
				continue;
			}
			else if ( j >= t1->tableSize )
				break;

			t1->entries[j].GetS1E1(jS1,jE1);
			t1->entries[j].GetS2E2(jS2,jE2);

			if ( !t1->entries[j].IsInUse() ) continue;			

			if ( t1->JoinForDeletedParalogs(i,j,1) )
				break;
											
		} while( t1->Intersec(i,jS1,jE1,1) != 0 );
	}

	for ( i=0 ; i<t2->tableSize ; i++ )
	{
		t2->entries[i].GetS2E2(iS2,iE2);

		j = i;
		// check for all the entries the come after i
		do {
			j++;
			if ( (j >= t2->tableSize) && (iS2 > iE2) )
			{
				j = -1;
				continue;
			}
			else if ( j >= t2->tableSize )
				break;

			t2->entries[j].GetS1E1(jS1,jE1);
			t2->entries[j].GetS2E2(jS2,jE2);

			if ( !t2->entries[j].IsInUse() ) continue;			

			if ( t2->JoinForDeletedParalogs(i,j,2) )
				break;
											
		} while( t2->Intersec(i,jS2,jE2,2) != 0 );
	}
	
	t1->SortBy(1);
	t2->SortBy(2);

	for ( i=0 ; i<t1->tableSize ; i++ )
	{
		if ( !t1->entries[i].IsInUse() ) continue;

		GENOME_TYPE S,E;
		t1->entries[i].GetS1E1(S,E);

		file1 << S << "\t" << E << "\t" << t1->GetLength(S,E,1) << "\t" << t1->entries[i].GetLabel() << endl;
	}

	for ( i=0 ; i<t2->tableSize ; i++ )
	{
		if ( !t2->entries[i].IsInUse() ) continue;

		GENOME_TYPE S,E;
		t2->entries[i].GetS2E2(S,E);

		file2 << S << "\t" << E << "\t" << t2->GetLength(S,E,2) << "\t" << t2->entries[i].GetLabel() << endl;
	}

	delete t1;
	delete t2;
}

//*************************************************************/

template <class GENOME_TYPE>
bool Table<GENOME_TYPE>::JoinForDeletedParalogs(unsigned long i, unsigned long j , int orgNum)
{
	GENOME_TYPE iS, iE, jS, jE ; 

	if ( orgNum == 1 )
	{
        this->entries[i].GetS1E1(iS,iE);
        this->entries[j].GetS1E1(jS,jE);
	}
	else
	{
        this->entries[i].GetS2E2(iS,iE);
        this->entries[j].GetS2E2(jS,jE);
	}


    if ( (this->Intersec(i,jS,jE,orgNum) > 0.7) || (this->Intersec(j,iS,iE,orgNum) > 0.7)  ) 
	{

		string st = this->entries[i].GetLabel() ;

		if ( iS > iE ) // entry i is circular
		{
			if ( jS > jE ) // entry j is circular
			{
				if ( iS < jS ) jS = iS;
				if ( iE > jE ) jE = iE;
			}
			else // entry j is not circular
			{
				if ( jE >= iS ) //  . )		[ ( ] .
					jE = iE ;
				else			 //  . [ ) ]	    ( .
					jS = iS;
			}
		}
		else  // entry i is not circular
		{
			if ( jS < jE ) // entry j is not circular
			{
				if ( iS < jS ) jS = iS;
				if ( iE > jE ) jE = iE;
			}
			else // entry j is circular
			{
				if ( iE >= jS ) //  . ]		( [ ) .
					jS = iS ;
				else			 //  . ( ] )	    [ .
					jE = iE;
			}
		}
		

		st += ":";
		st.append(this->entries[j].GetLabel());			

		if ( orgNum == 1 )
            this->entries[j].InseretEntry(jS,jE,0,0,this->entries[j].GetSign(),st);
		else
			this->entries[j].InseretEntry(0,0,jS,jE,this->entries[j].GetSign(),st);
		
		this->DeleteEntry(i);

		return true;
		
	} // if	

	return false;
}

//*************************************************************/


#endif


